﻿using Hotline.KnowledgeBase.Notifies;
using Hotline.KnowledgeBase;
using Hotline.Push;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Push;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using MediatR;
using Hotline.Push.Notifies;
using Hotline.Share.Enums.Push;
using NetTaste;
using XF.Domain.Filters;
using System.Net;
using System.Text;

namespace Hotline.Api.Controllers
{
    public class PushMessageController : BaseController
    {
        private readonly IMessageRepository _messageRepository;
        private readonly IMapper _mapper;
        private readonly IPushDomainService _pushDomainService;
        /// <summary>
        /// 日志记录
        /// </summary>
        private readonly ILogger<PushMessageController> _logger;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="messageRepository"></param>
        /// <param name="mapper"></param>
        /// <param name="pushDomainService"></param>
        /// <param name="mediator"></param>
        public PushMessageController(IMessageRepository messageRepository, IMapper mapper, IPushDomainService pushDomainService, ILogger<PushMessageController> logger)
        {
            _messageRepository = messageRepository;
            _mapper = mapper;
            _pushDomainService = pushDomainService;
            _logger = logger;
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("add")]
        public async Task AddMessage([FromBody] MessageDto dto)
        {
            await _pushDomainService.PushAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 查询短信剩余数量
        /// </summary>
        /// <returns></returns>
        [HttpGet("query")]
        public async Task<string> GetAccountNum()
        {
            return await _pushDomainService.GetAccountNum();
        }

        #region 短信发送状态回调接口
        /// <summary>
        /// 短信发送状态回调接口
        /// </summary>
        /// <param name="account">短信回传账号</param>
        /// <param name="pswd">短信回传密码</param>
        /// <param name="msgid">短信中心短信待发送ID，同短信发送接口返回值ID</param>
        /// <param name="sendtime">短信发送时间</param>
        /// <param name="msgcount">发送短信使用的数量</param>
        /// <param name="state">发送状态：0:未发送  1:发送中  2:发送失败  3:发送成功</param>
        /// <param name="errormsg">错误消息</param>
        /// <param name="sfid">短信中心短信已发送ID</param>
        /// <param name="telnumall">手机号码</param>
        /// <param name="sign">短信签名</param>
        /// <returns></returns>
        [HttpPost("receiveobtain")]
        [AllowAnonymous]
        public async Task<OpenResponse> ReceiveObtain([FromForm] ReceiveMessageDto receiveMessageDto)
        {
            var result = await _pushDomainService.ReceiveObtain(receiveMessageDto);

            return OpenResponse.Ok(result);
        }
        #endregion

        #region 短信接收
        /// <summary>
        /// 短信接收
        /// </summary>
        /// <param name="account">短信回传账号</param>
        /// <param name="pswd">短信回传密码</param>
        /// <param name="msg">回复短信内容</param>
        /// <param name="mobile">回复手机号码</param>
        /// <param name="destcode">短信接收平台号码</param>
        /// <param name="motime">回复时间</param>
        /// <param name="fsfid">短信平台回复短信ID</param>
        /// <returns></returns>
        [HttpPost("receivesms")]
        [AllowAnonymous]
        public async Task<OpenResponse> ReceiveSms([FromForm] ReceiveMessageDto receiveMessageDto)
        {
            var result = await _pushDomainService.ReceiveSms(receiveMessageDto);

            return OpenResponse.Ok(result);
        }
        #endregion

        /// <summary>
        /// 查询短信
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpGet("getlist")]
        public async Task<PagedDto<MessageDataDto>> AllMessage([FromQuery] MessagePagedDto pagedDto)
        {
            var (total, items) = await _messageRepository
                    .Queryable()
                    .Includes(it => it.User)
                    .Includes(it => it.SystemOrganize)
                    .Includes(it => it.Order)
                    .WhereIF(pagedDto.PushBusiness.HasValue, d => d.PushBusiness == pagedDto.PushBusiness)
                    .WhereIF(pagedDto.Status.HasValue, d => d.Status == pagedDto.Status)
                    .WhereIF(!string.IsNullOrEmpty(pagedDto.Keyword), d => d.Name.Contains(pagedDto.Keyword!) || d.Content.Contains(pagedDto.Keyword!) || d.TelNumber.Contains(pagedDto.Keyword!) || d.User.Name.Contains(pagedDto.Keyword!) || d.SystemOrganize.OrgName.Contains(pagedDto.Keyword!))
                    .WhereIF(pagedDto.StartTime.HasValue, d => d.SendTime >= pagedDto.StartTime)
                    .WhereIF(pagedDto.EndTime.HasValue, d => d.SendTime <= pagedDto.EndTime)
                    .OrderByDescending(it => it.CreationTime)
                    .ToPagedListAsync(pagedDto.PageIndex, pagedDto.PageSize, HttpContext.RequestAborted);

            return new PagedDto<MessageDataDto>(total, _mapper.Map<IReadOnlyList<MessageDataDto>>(items));

        }

      
    }
}
